"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadNS = void 0;
const ytdl_core_muxer_by_darky_1 = __importDefault(require("ytdl-core-muxer-by-darky"));
const fluent_ffmpeg_1 = __importDefault(require("fluent-ffmpeg"));
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const os_1 = require("os");
const nanostores_cjs_1 = require("nanostores-cjs");
const repl_ns_1 = require("repl-ns");
const duration_1 = require("../src/duration");
const ffmpeg_static_1 = __importDefault(require("ffmpeg-static"));
const p_event_1 = __importDefault(require("p-event"));
const events_1 = require("events");
if (ffmpeg_static_1.default)
    process.env['FFMPEG_PATH'] = ffmpeg_static_1.default;
exports.downloadNS = (0, repl_ns_1.ns)('download', {
    async onDownload(state) {
        (0, exports.downloadNS)().renderDownloading();
        try {
            const temporaryFilePath = await (0, exports.downloadNS)().createTempFilePath();
            const ytDownloading = (0, exports.downloadNS)().youtubeDownload(state.url);
            process.nextTick(async () => {
                try {
                    for await (const [, downloadedBytes, totalBytes] of (0, events_1.on)(ytDownloading, 'progress')) {
                        (0, exports.downloadNS)().renderDownloading(`${(downloadedBytes / 1024 / 1024).toFixed(0)} of ${(totalBytes / 1024 / 1024).toFixed(0)} MB`);
                    }
                }
                catch { }
            });
            await (0, p_event_1.default)(ytDownloading
                .on('error', (0, exports.downloadNS)().renderErr)
                .pipe((0, exports.downloadNS)().writeTempFile(temporaryFilePath).on('error', (0, exports.downloadNS)().renderErr)), 'finish');
            if (state.startTime || state.endTime || state.resolution !== 'highest') {
                (0, exports.downloadNS)().renderProcessing();
                const ffmpegStream = await Promise.resolve((0, exports.downloadNS)().createFfmpeg(temporaryFilePath))
                    .then(ff => (state.startTime ? ff.setStartTime(state.startTime) : ff))
                    .then(ff => (state.endTime ? ff.setDuration((0, duration_1.durationNS)().calcDuration(state)) : ff))
                    .then(ff => (state.resolution === 'highest' ? ff : ff.size(`?x${state.resolution}`)));
                await (0, p_event_1.default)(ffmpegStream.saveToFile(state.path), 'end');
            }
            else {
                await (0, exports.downloadNS)().cpFile(temporaryFilePath, state.path);
            }
            (0, exports.downloadNS)().renderComplete();
        }
        catch (err) {
            (0, exports.downloadNS)().renderErr(err);
        }
    },
    async createTempFilePath() {
        return `${await (0, promises_1.mkdtemp)(`${(0, os_1.tmpdir)()}/ytdl-tui-`)}/video.mp4`;
    },
    youtubeDownload(url) {
        return (0, ytdl_core_muxer_by_darky_1.default)(url);
    },
    writeTempFile(path) {
        return fs_1.default.createWriteStream(path);
    },
    createFfmpeg(path) {
        return (0, fluent_ffmpeg_1.default)(path);
    },
    async cpFile(from, to) {
        await (0, promises_1.copyFile)(from, to);
    },
    renderComplete() {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'completed', payload: '' });
    },
    renderErr(err) {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'error', payload: err.message });
    },
    renderDownloading(payload = '') {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'downloading', payload });
    },
    renderProcessing() {
        (0, exports.downloadNS)().downloadStatus$.set({ status: 'processing', payload: '' });
    },
    downloadStatus$: (0, nanostores_cjs_1.atom)({
        status: 'nothing',
        payload: '',
    }),
});
